#include "qmybt.h"

#define Inherited QButton

#include <qdialog.h>
#include <qfontmet.h>
#include <qpainter.h>
#include <qdrawutl.h>
#include <qpixmap.h>
#include <qpmcache.h>
#include <qbitmap.h>
#include <qpainter.h>
#include <qkeycode.h>
#include <qaccel.h>
#include <qpmcache.h>
#include <ctype.h>

QMyButton::QMyButton( QWidget *parent, const char *name )
        : Inherited( parent, name )
{
    mouseIsOn = false;
    swallowed_window = None;
}

QMyButton::~QMyButton ()
{
  if (most_recent_pressed == this)
    most_recent_pressed = NULL;
}

QMyButton* QMyButton::most_recent_pressed = NULL;

/*!
  Draws the push button, except its label.
  \sa drawButtonLabel()
*/

const int extraMotifWidth = 10;
const int extraMotifHeight = 10;

void QMyButton::drawButton( QPainter *paint )
{
    register QPainter *p = paint;
    GUIStyle	gs = style();
    QColorGroup g  = colorGroup();
    int		x1, y1, x2, y2;

    rect().coords( &x1, &y1, &x2, &y2 );	// get coordinates

    int w = x2 + 1;
    int h = y2 + 1;

    p->setPen( g.foreground() );
    p->setBrush( QBrush(g.background(),NoBrush) );

    if ( gs == WindowsStyle ) {		// Windows push button
	if ( isDown() ) {
            qDrawWinButton( p, x1, y1, w, h, g, TRUE );
	} else {
            QBrush fill(white, Dense4Pattern );
	    if ( isToggleButton() && isOn() && isEnabled() ) {
		qDrawWinButton( p, x1, y1, x2-x1+1, y2-y1+1, g, TRUE, &fill);
	    } else {
                if (mouseIsOn)
                    qDrawWinButton( p,x1,y1,x2-x1+1,y2-y1+1,g,isOn(),&fill);
                else
                    qDrawWinButton( p, x1, y1, x2-x1+1, y2-y1+1, g, isOn());
	    }
	}
    } else if ( gs == MotifStyle ) {		// Motif push button
	QBrush fill;
	if ( isDown() )
	    fill = QBrush( g.mid() );
	else if ( isOn() )
	    fill = QBrush( g.mid(), Dense4Pattern );
	else if (mouseIsOn)
            fill = QBrush( white, Dense4Pattern );
        else
	    fill = QBrush( g.background() );

	qDrawShadePanel( p, x1, y1, x2-x1+1, y2-y1+1, g, isOn() || isDown(),
			 2, &fill );
    }
    if ( p->brush().style() != NoBrush )
	p->setBrush( NoBrush );

    drawButtonLabel( p );

    if ( hasFocus() ) {
	if ( style() == WindowsStyle ) {
	    p->drawWinFocusRect( x1+3, y1+3, x2-x1-5, y2-y1-5, 
				 backgroundColor() );
	} else {
	    p->setPen( black );
	    p->drawRect( x1+3, y1+3, x2-x1-5, y2-y1-5 );
	}
    }
}

void QMyButton::drawButtonLabel( QPainter *paint )
{
    register QPainter *p = paint;

    QRect r = rect();
    int x, y, w, h, h1;
    r.rect( &x, &y, &w, &h );
    if ( (isDown() || isOn()) && style() == WindowsStyle ) {
        // shift pixmap/text
        x++;
        y++;
    }
    if (text() && strlen(text()))
        h1 = h - QFontMetrics(font()).height();
    else
        h1 = h;
    x += 2;  y += 2;  w -= 4;  h -= 4; h1 -= 4;
    qDrawItem( p, style(), x, y, w, h1,
               AlignCenter|ShowPrefix,
               colorGroup(), isEnabled(),
               pixmap(), 0);
    qDrawItem( p, style(), x, y, w, h,
               AlignHCenter|AlignBottom|ShowPrefix,
               colorGroup(), isEnabled(),
               0, text() );
}

/*
  Internal function that returns the shortcut character in a string.
  Returns zero if no shortcut character was found.
  Example:
    shortcutChar("E&xit") returns 'x'.
*/

static int shortcutChar( const char *str )
{
    const char *p = str ? strchr(str, '&') : 0;
    while ( p && *p && p[1] == '&' )
	p = strchr( p+2, '&' );
    return (p && *p && p[1] && p[1] != '&') ? p[1] : 0;
}

void QMyButton::setText( const char *text )
{
    QButton::setText(text);
}

void QButton::setText( const char *text )
{
    if ( btext == text )
	return;
    int oldAccelChar = shortcutChar(btext);
    int newAccelChar = shortcutChar(text);
    btext = text;
    if ( autoresize )
	adjustSize();
    if ( oldAccelChar && !newAccelChar && !accel() )
	setAccel( 0 );
    if ( newAccelChar )
	setAccel( ALT+toupper(newAccelChar) );
    repaint( FALSE );
}

void QMyButton::setPixmap( const QPixmap &pixmap )
{
    QButton::setPixmap(pixmap);
}

void QButton::setPixmap( const QPixmap &pixmap )
{
    int  w, h;
    if ( bpixmap ) {
	w = bpixmap->width();
	h = bpixmap->height();
    } else {
	bpixmap = new QPixmap;
	CHECK_PTR( bpixmap );
	w = h = -1;
    }
    bool sameSize = w == bpixmap->width() && h == bpixmap->height();
    *bpixmap = pixmap;
    if ( bpixmap->depth() == 1 && !bpixmap->mask() )
	bpixmap->setMask( *((QBitmap *)bpixmap) );
    if ( autoresize && !sameSize )
	adjustSize();
    repaint( FALSE );
}

void QMyButton::mousePressEvent(QMouseEvent *e)
{
    if (isDown())
        return;

    bool hit = hitButton(e->pos());
    if (hit) {
        most_recent_pressed = this;
        last_button = e->button();
        setDown( TRUE );
//      if (style() == WindowsStyle && swallowed_window != None)
//          XMoveWindow(qt_xdisplay(), swallowed_window, 4, 4);
        repaint(FALSE);
        emit pressed();
    }
}

void QMyButton::mouseReleaseEvent(QMouseEvent *e)
{
    if (!isDown()) {
        last_button = 0;
        releaseMouse();
        return;
    }
    bool hit = hitButton(e->pos());
//    if (style() == WindowsStyle && swallowed_window != None)
//        XMoveWindow(qt_xdisplay(), swallowed_window, 3, 3);
    setDown(FALSE);
    if (hit) {
        repaint( FALSE );
        emit released();
        emit clicked(); 
    } else {
        repaint();
        emit released();
    }
    last_button = 0;
    releaseMouse();
}

void QMyButton::enterEvent( QEvent *)
{
    if (mouseIsOn)
        return;
    setMouseIsOn(true);
}

void QMyButton::leaveEvent( QEvent *)
{
    if (!mouseIsOn)
        return;
    setMouseIsOn(false);
}

void QMyButton::setButtonOn(bool flag)
{
    setOn(flag);
}

void QMyButton::setToggle(bool flag)
{
    setToggleButton(flag);
}

void QMyButton::setMouseIsOn(bool flag)
{
    mouseIsOn = flag;
    repaint();
}
